unit INI_File;

interface

uses
 Windows, Forms, SysUtils, StdCtrls, StrUtils, Math, String_Func, Registry_Func,
 RealGlobals, shlobj, ShFolder;

Function GetStr(AppName, KeyName: String;
                RemoveHotKey    : Boolean = True;
                Filename        : String = ''): String;
Function GetDefaultStr(AppName, KeyName: String;
                       DefaultString   : String;
                       RemoveHotKey    : Boolean = True;
                       Filename        : String = ''): String;
Function WriteStr(Section, Key, Content: String): Integer;
  
Function GetSetting(AppName, KeyName: String; FileName: String = ''): String;
Function WriteSetting(AppName, KeyName, Data: String; FileName: String = ''): Integer;
Function LoadIntFromINI(Section, Topic: String;
                        MinVal, MaxVal: Integer;
                        var TargetInt : Integer;
                        FileName      : String = ''): Boolean;
Function LoadCardinalFromINI(Section, Topic: String;
                             MinVal, MaxVal: Integer;
                             var Target    : Cardinal;
                             FileName      : String = ''): Boolean;
Function LoadSingleFromINI(Section, Topic: String;
                           MinVal, MaxVal: Single;
                           var Target    : Single;
                           FileName      : String = ''): Boolean;
Function LoadDoubleFromINI(Section, Topic: String;
                           MinVal, MaxVal: Double;
                           var Target    : Double;
                           FileName      : String = ''): Boolean;
Function LoadTimeFromINI(Section, Topic: String;
                         var Target    : TDateTime;
                         FileName      : String = ''): Boolean;
Function LoadBoolFromINI(Section, Topic: String;
                         var Target    : Boolean;
                         FileName      : String = ''): Boolean;
Function ReplaceDotWithLocalDecimal(S: String): String;
Function ReplaceCommaWithDot(S: String): String;
Function ReplaceCommaORlocalDecimalWithDot(S: String): String;
Function ReplaceLocalDecimalWithDot(S: String): String;
Function ReplaceCommaORDotWithlocalDecimal(S: String): String;

Function GetMyDocumentsFolderName: string;

implementation

Function GetStr(AppName, KeyName: String;
                RemoveHotKey    : Boolean = True;
                Filename        : String = ''): String;
var
 Buffer: Array[0..1024] of Char;
 len   : Integer;
 S     : String;
begin
 If FileName = '' then Filename:= Txt_Name;
 try
  If KeyName = '' then //read complete section. Is "Buffer" big enough?
   len:= GetPrivateProfileString(pChar(AppName), nil, '', Buffer,
                                SizeOf(Buffer), PChar(Filename))
  else
   len:= GetPrivateProfileString(pChar(AppName), pChar(KeyName), '', Buffer,
                                SizeOf(Buffer), PChar(Filename));
  if len = 0 then
   S:= StringNotFound
  else
   SetString(S, Buffer, Len);

  //Delphi automatically generates the & for menus but not for buttons
  if RemoveHotKey then
  begin
   //Remove Ampersand "&"
   //Flags are [rfReplaceAll, rfIgnoreCase]
   S:= StringReplace(S, '&', '', [rfReplaceAll]);
  end;
  Result:= Trim(S);
 except
  Result:= StringNotFound;
 end;
end;

Function GetDefaultStr(AppName, KeyName: String;
                       DefaultString   : String;
                       RemoveHotKey    : Boolean = True;
                       Filename        : String = ''): String;
begin
 Result:= GetStr(AppName, KeyName, RemoveHotKey, Filename);
 If Result = StringNotFound then Result:= DefaultString
end;

Function WriteStr(Section, Key, Content: String): Integer;
var
 Success: LongBool;
begin
 If (Key = '') then //delete key
  Success:= WritePrivateProfileString(Pchar(Section),
                                     PChar(Nil),
                                     PChar(Content),
                                     PChar(TXT_Name))
 else If (Content = '') then //delete entry
  Success:= WritePrivateProfileString(Pchar(Section),
                                     PChar(Key),
                                     PChar(Nil),
                                     PChar(TXT_Name))
 else //write data
  Success:= WritePrivateProfileString(Pchar(Section),
                                     PChar(Key),
                                     PChar(Content),
                                     PChar(TXT_Name));
  If Success then
   Result:= 0
  else
   Result:= GetLastError;
 end;

Function GetSetting(AppName, KeyName: String; FileName: String = ''): String;
//Be careful when reading complete sections. Increase Buffer size if needed
var
 Buffer: Array[0..2048] of Char;
 len   : DWord;
begin
 If FileName = '' then Filename:= INI_Name;
 If KeyName = '' then
  len:= GetPrivateProfileString(pChar(AppName), nil, '_', Buffer,
                                SizeOf(Buffer), pChar(Filename))
 else
  len:= GetPrivateProfileString(pChar(AppName), pChar(KeyName), '_', Buffer,
                                SizeOf(Buffer), pChar(Filename));
 SetString(Result, Buffer, Len);
 If Result = '_' then Result:= '';
 Result:= Trim(Result);
end;

Function WriteSetting(AppName, KeyName, Data: String; FileName: String = ''): Integer;
var
 Success: LongBool;
begin //Write to APP.ini
 If FileName = '' then FileName:= INI_Name;
 If (KeyName = '') and (Data = '') then //Delete Section
  Success:= WritePrivateProfileString(Pchar(AppName),
                                  PChar(Nil),
                                  PChar(Nil),
                                  pchar(FileName))
 else If (KeyName = '') then //Delete Key
  Success:= WritePrivateProfileString(Pchar(AppName),
                                  PChar(Nil),
                                  PChar(Trim(Data)),
                                  pchar(FileName))
 else If (Data = '') then //Delete Entry
  Success:= WritePrivateProfileString(Pchar(AppName),
                                  PChar(KeyName),
                                  PChar(Nil),
                                  pchar(FileName))
 else //write data
  Success:= WritePrivateProfileString(Pchar(AppName),
                                  PChar(KeyName),
                                  PChar(Trim(Data)),
                                  pchar(FileName));
 If Success then
  Result:= 0
 else
  Result:= GetLastError;
End;

//32-Bit Integer
Function LoadIntFromINI(Section, Topic: String;
                        MinVal, MaxVal: Integer;
                        var TargetInt : Integer;
                        FileName      : String = ''): Boolean;
var
 S               : String;
 DefaultVal, Code: Integer;
 DBL             : Double;
begin
 DefaultVal:= EnsureRange(TargetInt, MinVal, MaxVal);
 If FileName ='' then FileName:= INI_Name;
 If MinVal > MaxVal then MinVal:= MaxVal;
 S:= GetSetting(Section, Topic, FileName);
 If S > '' then
 begin
  try
   Val(ReplaceCommaORlocalDecimalWithDot(S), DBL, Code);
   If Code <> 0
    then TargetInt:= DefaultVal
    else
     TargetInt:= Round(DBL);

   Result:= True;
  except
   TargetInt:= DefaultVal;
   Result:= False;
  end;
 end
 else
 begin
  TargetInt:= DefaultVal;
  Result:= True;
 end;

 TargetInt:= EnsureRange(TargetInt, MinVal, MaxVal);
end;

//32-Bit Word
Function LoadCardinalFromINI(Section, Topic: String;
                             MinVal, MaxVal: Integer;
                             var Target    : Cardinal; //Long Word
                             FileName      : String = ''): Boolean;
var
 S         : String;
 DefaultVal: Cardinal;
 DBL       : Double;
 Code      : Integer;
begin
 Result:= False;
 DefaultVal:= Target;
 If FileName ='' then FileName:= INI_Name;
 If MinVal > MaxVal then MinVal:= MaxVal;
 S:= GetSetting(Section, Topic, FileName);
 If S > '' then
 begin
  try
   Val(ReplaceCommaORlocalDecimalWithDot(S), DBL, Code);
   If Code <> 0
    then Target:= DefaultVal
    else Target:= Abs(Round(DBL));

   Result:= True;
  except
   Target:= DefaultVal;
  end;
 end
 else
  Target:= DefaultVal;

 Target:= EnsureRange(Target, MinVal, MaxVal);
end;

Function LoadSingleFromINI(Section, Topic: String;
                           MinVal,
                           MaxVal        : Single;
                           var Target    : Single;
                           FileName      : String = ''): Boolean;
var
 S         : String;
 Code      : Integer;
 DefaultVal: Double;
begin
 DefaultVal:= Target;
 Result:= False;
 If MinVal > MaxVal then MinVal:= MaxVal;
 S:= GetSetting(Section, Topic);
 If S > '' then
 begin
  Val(ReplaceCommaORlocalDecimalWithDot(S), Target, Code);
  Result:= True;
  If Code <> 0 then Target:= DefaultVal;
 end
 else
  Target:= DefaultVal;

 EnsureRange(Target, MinVal, MaxVal);
end;

//32-Bit Double precision
Function LoadDoubleFromINI(Section, Topic: String;
                           MinVal,
                           MaxVal        : Double;
                           var Target    : Double;
                           FileName      : String = ''): Boolean;
var
 S         : String;
 Code      : Integer;
 DefaultVal: Double;
begin
 DefaultVal:= Target;
 Result:= False;
 If MinVal > MaxVal then MinVal:= MaxVal;
 S:= GetSetting(Section, Topic);
 If S > '' then
 begin
  Val(ReplaceCommaORlocalDecimalWithDot(S), Target, Code);
  Result:= True;
  If Code <> 0 then Target:= DefaultVal;
 end
 else
  Target:= DefaultVal;

 EnsureRange(Target, MinVal, MaxVal);
end;

//32-Bit Double precision
Function LoadTimeFromINI(Section, Topic: String;
                         var Target    : TDateTime;
                         FileName      : String = ''): Boolean;
var
 S         : String;
 x, Code,
 St, Mi    : Integer;
 DefaultVal: TDateTime;
begin
 DefaultVal:= Target;
 Result:= False;
 S:= GetSetting(Section, Topic);
 If S > '' then
 begin
  x:= Instring(0, S, ':');
  If x > 0 then ///Minute found
  begin
   Val(Copy(S, 1, x - 1), St, Code);
   Val(Copy(S, x + 1, 2), Mi, Code);
  end
  else  //nur Stunde
   begin
    Val(S, St, Code);
    Mi:= 0;
   end;

  Target:= EncodeTime(St, Mi, 0, 0);
  Result:= True;
  If Code <> 0 then Target:= DefaultVal;
 end
 else
  Target:= DefaultVal;
end;

Function LoadBoolFromINI(Section, Topic: String;
                         var Target    : Boolean;
                         FileName      : String = ''): Boolean;
var
 S         : String;
 DefaultVal: Boolean;
begin
 DefaultVal:= Target;
 Result:= True;
 S:= GetSetting(Section, Topic);
 If S > '' then
  Target:= UpperCase(S) = 'TRUE'
 else
  Target:= DefaultVal;
end;

Function ReplaceDotWithLocalDecimal(S: String): String;
 //Setzt einen . fr ein , ein
begin
 If length(S) > 0
  //then Result:= StringReplace(S, '.', GetSDecimal, [rfReplaceAll])
  then Result:= StringReplace(S, '.', DecimalSeparator, [rfReplaceAll])
  else Result:= S;
End;

Function ReplaceCommaWithDot(S: String): String;
 //Setzt einen . fr ein , ein
begin
 If length(S) > 0
  then Result:= StringReplace(S, ',', '.', [rfReplaceAll])
  else Result:= S;
End;

Function ReplaceCommaORlocalDecimalWithDot(S: String): String;
 //Setzt einen . fr ein , oder <local decimal> ein
begin
 If length(S) > 0
  then
   begin
    Result:= StringReplace(S, ',', '.', [rfReplaceAll]);
    Result:= StringReplace(Result, GetSDecimal, '.', [rfReplaceAll]);
   end
  else Result:= S;
End;

Function ReplaceLocalDecimalWithDot(S: String): String;
 //Setzt einen . fr den <local decimal> ein
begin
 If length(S) > 0
  then Result:= StringReplace(S, GetSDecimal, '.', [rfReplaceAll])
  else Result:= S;
End;

Function ReplaceCommaORDotWithlocalDecimal(S: String): String;
 //Setzt das <local decimal> fr einen . oder , ein
begin
 If length(S) > 0
  then
   begin
    Result:= StringReplace(S, ',', GetSDecimal, [rfReplaceAll]);
    Result:= StringReplace(Result, '.', GetSDecimal, [rfReplaceAll]);
   end
  else Result:= S;
End;

Function GetMyDocumentsFolderName: string;
var
 R   : Bool;
 path: Array[0..Max_Path] of Char;
begin
 R:= ShGetSpecialFolderPath(0, path, CSIDL_Personal, False) ;
 if not R
  then Result:= ''
  else Result:= Path;
end;

end.
