unit String_Func;

interface

uses
 StrUtils, SysUtils, Windows, Graphics, Classes, StdCtrls, Grids, Registry_Func;

Type
 ValignType = (ValignTop, ValignCenter, ValignBottom);

 TStringGridHack = class(TStringGrid)
  protected
   procedure DeleteRow(ARow: Longint); reintroduce;
   procedure InsertRow(ARow: Longint);
  end;

//Reduce String to a certain length by substituting "..."
Function Minced(PathToMince: String; MaxLen: Integer): String;

Function ExtractFileNameWithoutExt(Filename: String): String;

//Pad a string to a certain length. Example pad "34" to 6 digits with "0" on left side
// ==> 000034
Function PadString(S       : String;
                   PadChar : Char;
                   MaxLen  : Integer;
                   LeftSide: Boolean = True): String;

//Remove Ampersand "&". Flags are [rfReplaceAll, rfIgnoreCase]
Function ReducedStr(S: String): String;

  {replace %<x> by %s  ==> String = Hallo
                by %d  ==> Integer = 12345
                by %g  ==> General Real ==> shortest possible number
                by %x  ==> Hexadecimal = FF2A
                by %e  ==> Scientific Real = 1.23456780000000E+004
                by %n  ==> Formatted Real = 12.345,68 using "local"
                by %m  ==> Formatted Real as Currency = 12.345,68 using "local"
   Ein evtl. bergebener Genauigkeitsbezeichner wird ebenfalls benommen.
   Genauigkeit "%0.2n" ergibt krzest mglichen String mit 2 Nachkommastellen

  Zero padding:
   The width value dictates the output size with blank padding to the left
   Note the <> characters are added to show formatting

   ShowMessage(Format('Padded decimal    = <%7d>', [1234]));

   With the '-' operator, the data is left justified:
   ShowMessage(Format('Justified decimal = <%-7d>', [1234]));

   The precision value forces 0 padding to the desired size:
   ShowMessage(Format('0 padded decimal  = <%.6d>', [1234]));

  Beispielaufrufe:
   1. ResolveFmtString('%1 Datenstze in %2 mS gelesen, ['n', 'n']);

   2. ResolveFmtString('%1 Datenstze in %2 mS gelesen, ['n8.2', 'n']);

  Ergebnisse:
   1. '%n Datenstze in %n mS gelesen'
   2. '%n8.2 Datenstze in %n mS gelesen'
  }
Function ResolveFmtString(FmtStr: String; FmtArray: Array of String): String;

//search for a sub string
Function InString(StartPos: Integer; MainStr: String; SubStr: String): Integer;
Function RemoveLeftSlashes(S: String): String; //Remove possible left slashes
Function RemoveRightSlashes(S: String): String; //Remove possible right slashes
Procedure AddRightSlash(var S: String); //Add possible right slash
Function Trim0(S: String): String;
Procedure ReformatIPstring(var Source: String; IPV: Integer = 4);
Function IntToBin(Value     : Integer;
                  NumBits   : Integer = 32;
                  WithSpaces: Boolean = True): string;
procedure Sto_DrawCellText(const Canvas              : TCanvas;
                           const Rect                : TRect;
                           const Text                : String;
                           const BackColor, TextColor: TColor;
                           const Alignment           : TAlignment);
Procedure RotatedText(Rotation, x, y: Integer;
                      aText         : String;
                      aCanvas       : TCanvas);
//Vertikale Textausgabe in den Zellen eines StringGrid
procedure StringGridRotateText90Out(Grid          : TStringGrid;
                                    ARow, ACol    : Integer;
                                    Font_Size     : Integer;
                                    Rect          : TRect;
                                    FontColor     : TColor;
                                    HAlignment    : TAlignment;
                                    VAlignment    : ValignType;
                                    Height2Txt,
                                    Width2Txt     : Boolean);

Function FileTime2String(Tm: TFileTime): String;

Function MakeFileDateTimeString_SYSTEM(var stLocal: TSystemTime): String;

implementation

Procedure TStringGridHack.DeleteRow(ARow: Longint);
var
 GemRow: Integer;
begin
 GemRow:= Row;
 if RowCount > FixedRows + 1 then
  inherited DeleteRow(ARow)
 else
  Rows[ARow].Clear;
 if GemRow < RowCount then Row:= GemRow;
end;

Procedure TStringGridHack.InsertRow(ARow: Longint);
var
 GemRow: Integer;
begin
 GemRow:= Row;
 while ARow < FixedRows do Inc(ARow);
 RowCount:= RowCount + 1;
 MoveRow(RowCount - 1, ARow);
 Row:= GemRow;
 Rows[Row].Clear;
end;

Function Minced(PathToMince: String; MaxLen: Integer): String;
 {Example:
   C:\Program Files\Delphi\DDrop\TargetDemo\main.pas
   C:\Program Files\..\main.pas  }
Var
 sl          : TStringList;
 sHelp, sFile: String;
 iPos        : Integer;

 Function ShortFileName(FileName: String): String;
 Var
  aTmp: Array[0..255] Of Char;
 Begin
  If Not FileExists(FileName)
  Then
   Result := ''
  Else
   If GetShortPathName(PChar (FileName), aTmp, SizeOf (aTmp) - 1) = 0 Then
    Result:= FileName
   Else
    Result:= StrPas(aTmp);
 End;

 Function LongFileName(ShortName: String): String;
 Var
  SR: TSearchRec;
 Begin
  Result:= '';
  If (pos ('\\', ShortName) + pos ('*', ShortName) +
      pos ('?', ShortName) <> 0) Or Not FileExists(ShortName) Then
   Exit; //ignore NetBIOS name, joker chars and invalid file names

  While FindFirst(ShortName, faAnyFile, SR) = 0 Do
  Begin
   //next part as prefix
   Result := '\' + SR.Name + Result;
   SysUtils.FindClose(SR); //the SysUtils, not the WinProcs procedure!
   //directory up (cut before '\')
   ShortName := ExtractFileDir (ShortName);
   If length (ShortName) <= 2 Then
    Break; //ShortName contains drive letter followed by ':'
  End;
  Result:= ExtractFileDrive (ShortName) + Result;
 end;

Begin
 sHelp:= PathToMince;
 iPos:= Pos('\', sHelp);
 If iPos = 0
 Then
  Result:= PathToMince
 Else
  Begin
   sl := TStringList.Create;
   //Decode string
   While iPos <> 0 Do
   Begin
    sl.Add(Copy(sHelp, 1, (iPos - 1)));
    sHelp:= Copy(sHelp, (iPos + 1), Length(sHelp));
    iPos:= Pos('\', sHelp);
   End;
   If sHelp <> '' Then sl.Add(sHelp);
   //Encode string
   sFile:= sl[sl.Count - 1];
   sl.Delete(sl.Count - 1);
   Result:= '';
   While (Length(Result + sFile) < MaxLen) And (sl.Count <> 0) Do
   Begin
    Result:= Result + sl[0] + '\';
    sl.Delete(0);
   End;
   If sl.Count = 0
   Then
    Result:= Result + sFile
   Else
    Result:= Result + '..\' + sFile;

   sl.Free;
  End;
end;

Function ExtractFileNameWithoutExt(Filename: String): String;
var
 S: String;
begin
 try
  S:= ExtractFileName(FileName);
  Result:= LeftStr(S, Instring(0, S, '.') -1);
 except
  Result:= Filename;
 end;
end;

Function PadString(S       : String;
                   PadChar : Char;
                   MaxLen  : Integer;
                   LeftSide: Boolean = True): String;
begin
 While Length(S) < MaxLen do
  If LeftSide then
   S:= PadChar + S
  else
   S:= S + PadChar;
 Result:= S;
end;

Function ReducedStr(S: String): String; // Removes the & from a string
begin
 //Remove Ampersand "&". Flags are [rfReplaceAll, rfIgnoreCase]
 Result:= StringReplace(S, '&', '', [rfReplaceAll]);
End;

Function ResolveFmtString(FmtStr: String; FmtArray: Array of String): String;
var
 x, i: Integer;
 S   : String;
begin
 S:= FmtStr;
 i:= 0;
 x:= 0;
 While i <= High(FmtArray) do
 begin
  x:= InString(x + 1, S, '%' + IntToStr(i + 1));
  If x > 0 then
  begin
   If x + 2 < Length(S)
    then S:= LeftStr(S, x) + FmtArray[i] + MidStr(S, x + 2, Length(S))
    else S:= LeftStr(S, x) + FmtArray[i];
  end;
  Inc(i);
 end;
 Result:= S;
end;

Function InString(StartPos: Integer; MainStr: String; SubStr: String):Integer;
var
 x: Integer;
 S: String;
begin
 if StartPos < 1 then StartPos:=1;
 S:=copy(MainStr, StartPos, length(MainStr));
 x:=pos(SubStr, S);
 if x = 0 then
  Result:= 0
 else
  Result:= StartPos - 1 + x;
end;

Function RemoveLeftSlashes(S:String):String; //Remove possible left slashes
var
 SS: String;
begin
 SS:= S;
 While LeftStr(SS, 1) = '\' do SS:= Midstr(SS, 2, Length(SS));
 Result:= SS;
End;

Function RemoveRightSlashes(S:String):String; //Remove possible right slashes
var
 SS: String;
begin
 SS:= S;
 While StrUtils.RightStr(SS, 1) = '\' do SS:= LeftStr(SS, Length(SS) -1);
 Result:= SS;
End;

Procedure AddRightSlash(var S: String);
begin
 If S[length(S)] <> '\' Then S:= S + '\';
End;

Function Trim0(S: String): String; //Extract the string part up to the #0 char
var
 x: Integer;
begin
 Result:= ''; //Default
 If length(S) > 0 then
 begin
  x:= 1;
  While x <= length(S) do
  begin
   If S[x] = #0 then
   begin
    Result:= LeftStr(S, x - 1);
    Break;
   end;
   Inc(x);
  end;
 end;
end;

Procedure ReformatIPstring(var Source: String; IPV: Integer = 4);
{Input:
   Source = Sorce string. Example "123. 56.  9.0"
   IPV    = 4 or 6. Default is 4
 Result:
   Source = Sorce string. Example "123.56.9.0"

 Missing data is supplemented by "000"
 Superfluous data is truncated
 }
var
 i, x, d: Integer;
begin
 //remove spaces
 x:= Instring(0, Source, ' ');
 While x > 0 do
 begin
  Source:= Copy(Source, 1, x -1) + Copy(Source, x +1, Length(Source));
  x:= Instring(0, Source, ' ');
 end;

 //first character must not be a dot
 While (Source > '') and (Source[1] = '.') do Delete(Source, 1, 1);

 //last character must not be a dot
 While (Source > '') and (Source[Length(Source)] = '.') do
  Delete(Source, Length(Source), 1);

 //count dots
 i:= 0;
 x:= 1;
 Repeat
  x:= Instring(x +1, Source, '.');
  If x > 0 then inc(i);
 until x = 0;

 If Source[Length(Source)] = '.' then
 begin
  Delete(Source, Length(Source), 1);
  Dec(i);
 end;

 //Missing data is supplemented by ".0"
 While i < (IPV -1) do
 begin
  Source:= Source + '.0';
  Inc(i);
 end;

 //Superfluous data is truncated
 While i > (IPV -1) do
 begin
  d:= 0;
  //search for last dot
  x:= Instring(0, Source, '.');
  While x > 0 do
  begin
   d:= x;
   x:= Instring(d + 1, Source, '.');
  end;
  Source:= Copy(Source, 1, d -1);
  Dec(i);
 end;
 If Source[Length(Source)] = '.' then Delete(Source, Length(Source), 1);
end;

Function IntToBin(Value: Integer;
                  NumBits: Integer = 32;
                  WithSpaces: Boolean = True): string;
//returns a 32 character string by default
var
 i: Integer;
begin
 Result:= '';
 for i := 1 to NumBits do
 begin
  If (Value and (1 shl (NumBits - i)) <> 0)
   then Result:= Result + '1'
   else Result:= Result + '0';

  If WithSpaces then Result:= Result + ' ';
 end;
end;

////////////////////////////////////////////////////////////////
// draws the text for one single cell in a grid
procedure Sto_DrawCellText(const Canvas              : TCanvas;
                           const Rect                : TRect;
                           const Text                : String;
                           const BackColor, TextColor: TColor;
                           const Alignment           : TAlignment);
var
 iLeftBorder: Integer;
const
 BORDER_WIDTH = 2;
begin
 // calculate the left border
 iLeftBorder:= 0;
 case Alignment of
   taLeftJustify : iLeftBorder:= Rect.Left + BORDER_WIDTH;
   taRightJustify: iLeftBorder:= Rect.Right - BORDER_WIDTH -
                                                     Canvas.TextWidth(Text) -1;
   taCenter: iLeftBorder:= Rect.Left +
                       (Rect.Right - Rect.Left- Canvas.TextWidth(Text)) div 2;
 end;
 // set colors
 Canvas.Font.Color:= TextColor;
 Canvas.Brush.Color:= BackColor;
 // paint the text
 ExtTextOut(Canvas.Handle, iLeftBorder,
            Rect.Top + BORDER_WIDTH, ETO_CLIPPED or ETO_OPAQUE,
            @Rect, PChar(Text), Length(Text), nil);
end;

Procedure RotatedText(Rotation, x, y: Integer;
                      aText         : String;
                      aCanvas       : TCanvas);
Var 
 aFt     : LOGFONT;
 hOldFont: THandle; 
 //works with TImage only!
Begin 
 hOldFont:= aCanvas.Font.Handle; 
 Try
  aCanvas.Font.Name:= 'Arial'; // Muss TrueType sein
  GetObject(aCanvas.Font.Handle, SizeOf(aFt), @aFt); 
  aFt.lfEscapement:= 10 * Rotation;  // Rotate ist der Winkel in Grad 
  aFt.lfOrientation:= aft.lfEscapement;
  aFt.lfQuality:= ANTIALIASED_QUALITY; 
  aCanvas.Font.Handle:= CreateFontIndirect(aFt); 
  aCanvas.TextOut(x, y, aText); 
 Finally 
  DeleteObject(aCanvas.Font.Handle); 
  aCanvas.Font.Handle:= hOldFont; 
 End; 
End;

//Vertikale Textausgabe in den Zellen eines StringGrid
procedure StringGridRotateText90Out(Grid          : TStringGrid;
                                    ARow, ACol    : Integer;
                                    Font_Size     : Integer;
                                    Rect          : TRect;
                                    FontColor     : TColor;
                                    HAlignment    : TAlignment;
                                    VAlignment    : ValignType;
                                    Height2Txt,
                                    Width2Txt     : Boolean);
//Font.Name MUSS Truetype sein!!
var
 lf           : TLogFont;
 tf           : TFont;
 NewWidth,
 HorizP,
 VertP,
 OldFontSize,
 FontSize     : Integer;
 S            : String;
const
 DY = 2;
begin
 OldFontSize:= Grid.Canvas.Font.Size;
 S:= Trim(Grid.Cells[ACol, ARow]);
 If (S = '') then Exit;
 FontSize:= Font_Size;
 //wenn Schrift zu gro dann anpassen
 If Height2Txt then  //Horizontal hat Vorrang vor Vertical
 begin //set cell height to match string length
  If ((Grid.Canvas.TextWidth(S) + DY) >= Grid.RowHeights[aRow]) then
  begin
   Grid.RowHeights[aRow]:= Grid.Canvas.TextWidth(S) + (4 * DY);
   Rect.Bottom:= Rect.Top + Grid.RowHeights[aRow];
  end;
 end
 else
 begin //set font size to fit cell height
  NewWidth:= Grid.Canvas.TextWidth(S) + DY;
  If NewWidth > Grid.RowHeights[aRow] then
   FontSize:= Round(Font_Size * (Grid.RowHeights[aRow] / NewWidth));
 end;

 //Zellenbreite anpassen falls Schrifthhe > Zellenbreite ist
 If (Grid.Canvas.TextHeight(S) >= (Grid.ColWidths[ACol])) then
 begin
  Grid.ColWidths[ACol]:= Grid.Canvas.TextHeight(S);
  Rect.Right:= Rect.Left + Grid.ColWidths[ACol];
 end;

 //Font setzen
 Grid.Font.Color:= FontColor;
 Grid.Canvas.Font.Size:= FontSize;

 tf:= TFont.Create;
 try
  tf.Assign(Grid.Font);
  GetObject(tf.Handle, SizeOf(lf), @lf);
  lf.lfEscapement:= 900; //vertical
  lf.lfOrientation:= 0;
  tf.Handle:= CreateFontIndirect(lf);
  Grid.Canvas.Font.Assign(tf);
 finally
  tf.Free;
 end;

 //Rechteck fllen
 Grid.Canvas.FillRect(Rect);

 //Text ausgerichtet ausgeben
 if HAlignment = taLeftJustify
  then HorizP:= Rect.Left + 2
  else if HAlignment = taCenter
   then HorizP:= Rect.Left + Grid.ColWidths[ACol] div 2 - Grid.Canvas.Font.Size +
                 Grid.Canvas.Font.Size div 3
   else //taRightJustify
    HorizP:= Rect.Left + ((Rect.Right - Rect.Left - Grid.Canvas.Font.Size) div 2){- 2};

 If VAlignment = ValignTop
  then
   VertP:= Rect.Top + Grid.Canvas.TextWidth(S) + DY
  else If VAlignment = ValignCenter
   then
    VertP:= Rect.Bottom - ((Rect.Bottom - Rect.Top -
                           Grid.Canvas.TextWidth(S)) div 2) + DY
   else //ValignBottom
    VertP:= Rect.Bottom - DY;

 Grid.Canvas.TextRect(Rect, HorizP, VertP, Grid.Cells[ACol, ARow]);
 Grid.Canvas.Font.Size:= OldFontSize; //Restore original font size
end;

Function FileTime2String(Tm: TFileTime): String;
var
 SysTime: TSystemTime;
begin
FileTimeToSystemTime(Tm, SysTime);
If SysTime.wyear < 2000 then //time not available
 Result:= '00:00:00 not found'
else
 Result:= PadString(IntToStr(SysTime.wday), '0', 2) + '.' +
          PadString(IntToStr(SysTime.wmonth), '0', 2) + '.' +
          PadString(IntToStr(SysTime.wyear), '0', 4) + ' ' +
          PadString(IntToStr(SysTime.whour), '0', 2) + ':' +
          PadString(IntToStr(SysTime.wminute), '0', 2) + ':' +
          PadString(IntToStr(SysTime.wsecond), '0', 2) + ':' +
          PadString(IntToStr(SysTime.wmilliseconds), '0', 3);
end;

Function MakeFileDateTimeString_SYSTEM(var stLocal: TSystemTime): String;
begin
 If GetMD then
  Result:= Format('%2.2d.%2.2d.%4.4d %2.2d:%2.2d:%2.2d',
                  [stLocal.wMonth, stLocal.wDay, stLocal.wYear,
                   stLocal.wHour, stLocal.wMinute, stLocal.wSecond])
 else
  Result:= Format('%2.2d.%2.2d.%4.4d %2.2d:%2.2d:%2.2d',
                  [stLocal.wDay, stLocal.wMonth, stLocal.wYear,
                   stLocal.wHour, stLocal.wMinute, stLocal.wSecond]);
end;

end.
